'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createHookObject = exports.isTransitionHook = exports.isGlobalHook = exports.isLocalHook = exports.createTransitionHook = exports.createGlobalHook = exports.createLocalHook = undefined;

var _helpers = require('./helpers');

var TYPE_LOCAL = 'LOCAL';
var TYPE_GLOBAL = 'GLOBAL';
var TYPE_TRANSITION = 'TRANSITION';

var createHook = function createHook(type, action) {
  var _ref = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  var _ref$stopOnException = _ref.stopOnException;
  var stopOnException = _ref$stopOnException === undefined ? true : _ref$stopOnException;
  var _ref$executeIfPreviou = _ref.executeIfPreviousFailed;
  var executeIfPreviousFailed = _ref$executeIfPreviou === undefined ? false : _ref$executeIfPreviou;
  return {
    type: type,
    action: action,
    stopOnException: stopOnException,
    executeIfPreviousFailed: executeIfPreviousFailed
  };
};

var createLocalHook = exports.createLocalHook = function createLocalHook(name, options) {
  if (!(0, _helpers.isString)(name)) {
    throw new Error('Local hook name must be string');
  }
  return createHook(TYPE_LOCAL, name, options);
};

var createGlobalHook = exports.createGlobalHook = function createGlobalHook(callback, options) {
  if (!(0, _helpers.isFunction)(callback)) {
    throw new Error('Global hook callback must be function');
  }
  return createHook(TYPE_GLOBAL, callback, options);
};

var createTransitionHook = exports.createTransitionHook = function createTransitionHook(options) {
  return createHook(TYPE_TRANSITION, null, options);
};

var isLocalHook = exports.isLocalHook = function isLocalHook(hook) {
  return hook.type === TYPE_LOCAL && (0, _helpers.isString)(hook.action);
};

var isGlobalHook = exports.isGlobalHook = function isGlobalHook(hook) {
  return hook.type === TYPE_GLOBAL && (0, _helpers.isFunction)(hook.action);
};

var isTransitionHook = exports.isTransitionHook = function isTransitionHook(hook) {
  return hook.type === TYPE_TRANSITION;
};

var createHookObject = exports.createHookObject = function createHookObject(action) {
  if ((0, _helpers.isFunction)(action)) {
    return createGlobalHook(action);
  } else if ((0, _helpers.isString)(action)) {
    return createLocalHook(action);
  } else if (isLocalHook(action) || isGlobalHook(action) || isTransitionHook(action)) {
    return action;
  }
  throw new Error('Invalid argument received. Must be hook object, string or function.');
};